###
txt1 <- " Script for performing Bing search, normalizing, and"
txt2 <- " saving frequencies of (co-)occurrences of row & col keywords."
txt3 <- " (c) 12/2023 J. Mazanec"
# note that, for ease of comprehension, commands are less condensed 
# than they might be
###
# R Release 4.1.0.
#

cat("\n",txt1,"\n",txt2,"\n",txt3,"\n")

# install.packages("readr")
# install.packages("rvest")
# install.packages("stringr")

library(rvest)
library(readr)
library(stringr)

# load Bing search functions
source("bfunctions.R")
options(warn=-1) 				      # suppress warnings

### organize your data for new analysis ...
# define txt file for row keywords
rowfn <- 'cities6.txt'                    # 'countries6.txt'         
# define txt file for col keywords
colfn <- 'connotations4.txt'			# demo 
# define context for search
#context <- "+tourism+OR+tourist+OR+travel+OR+trip" # needed for domain size
context <- "trip"  # for a Bing page universe
# define txt file for row, col, and matrix frequency output, if desired
routfn <- 'cities6freq.txt'              	# demo
coutfn <- 'connot4freq.txt'              	# demo
moutfn <- 'cities6connot4freq.txt'       	# demo
nmoutfn <- 'cities6connot4freq_norm.txt' 	# demo
NGDfn <- 'cities6connot4NGD.txt'         	# demo       
ncofn <- 'cities6connot4nco.txt'         	# demo

### ... or retrieve stored resuls with 
# source("LoadCityData.R")

### capture row, col, and matrix keyword frequencies from Bing
# call gsearch.vec(rowfn, context="", outfn="", repet = 1)
rowkwfreq <- gsearch.vec(rowfn, context, routfn)
colkwfreq <- gsearch.vec(colfn, context, coutfn)
# call gsearch.mat(rowfn, colfn, context="", outfn="")
matkwfreq <- gsearch.mat(rowfn, colfn, context, moutfn)

### STEP 1
### row-normalize co-occurrence matrix with (=divide by) row keyword frequencies
# call row.normalize(rawmat, rowkwfreq, rowfn="", colfn="", outfn="")
kwfreq_norm <- row.normalize(matkwfreq, rowkwfreq, rowfn, colfn, nmoutfn)

# hierarchical clustering (complete linkage) may be applied to the 
# dissimilarity matrix derived from dist or from (1 - corr(co-occurrence matrix))
# note that row-normalization makes no difference for corr
# but still ignores popularity of col keywords 
windows()
dendro.cor(matkwfreq, rowfn)    # same as (kwfreqnorm, ...
windows()
dendro.dis(kwfreq_norm, rowfn)

### STEP 2
### compute a cell-normalized co-occurrence matrix by dividing keyword 
# intersection by keyword union of frequencies of co-occurrence
# call cell.normalized.prox(rowcol, rowfreq, colfreq, rowfn="", colfn="", outfn="")
nco <- cell.normalized.prox(matkwfreq, rowkwfreq, colkwfreq, rowfn, colfn, ncofn)

# hierarchical clustering on nco distances and (1 - cor) among cities
windows()
dendro.dis(nco, rowfn)          # too many small distances 
windows()
dendro.cor(nco, rowfn)		  # cor on small profile variations

### STEP 3
### compute the Normalized Google Distance NGD (Cilibrasi & Vitanyi, 2007)
# first we need a normalizing factor such as the overall frequency of the domain
# defined by the context 
N <- gsearch.sca(context)
# now we've got everything needed for computing the NGD
# call NGD(rowcol, rowfreq, colfreq, rowfn="", colfn="", outfn="")
NGDmat <- NGD(matkwfreq, rowkwfreq, colkwfreq, N, rowfn, colfn, NGDfn)

# NOTE: NGDmat may be transformed into inter-city distances and 
# analyzed with Multidimensional Scaling methods (e.g. with procedures 
# PROXIMITIES and ALSCAL of the SPSS Base System (square matrix, matrix-conditional);
# NGDmat may also be analyzed directly with Multidimensional Unfolding
# (e.g. with ALSCAL of SPSS for a rectangular matrix, matrix-conditional);
# see spss syntax file alscalNGD.sps for an example;

### apply hierarchical clustering to the dissimilarities derived from
# the correlations and distances among rows (cities) 
windows()
dendro.dis(NGDmat, rowfn)
windows()
dendro.cor(NGDmat, rowfn)

# if desired corr and dist among columns (connotations)
windows()
dendro.cor(t(NGDmat), colfn)
windows()
dendro.dis(t(NGDmat), colfn)

### STEP 4
# apply non-disjunctive hierarchical clustering
# write input and run CLIP.exe
run.clip(NGDmat,rowfn)

### STEP 5
### apply classical MDS to NGD distances among cities in R^2
windows()
mds.mds(NGDmat, rowfn, k=2) 
### apply Sammon mapping (uses classical MDS result for initialization)
windows()
mds.sam(NGDmat, rowfn, k=2)

# graphics.off()




